"use strict";
// @ts-nocheck
/**
 *
 * **/
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.filterDocs = exports.loadAndParseDoc = exports.getDocsByPath = exports.getDocMetaBySlug = exports.computeHrefForDoc = exports.getDocBySlug = void 0;
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const gray_matter_1 = __importDefault(require("gray-matter"));
const luxon_1 = require("luxon");
// import { BASE_PATH } from "./constants";
const fs_2 = require("./fs");
const formatters_1 = require("./formatters");
/**
 * Retrieve a markdown document from the `content` directory, parsed and ready to go
 * @param {string} slug slug of the file name to locate
 * @param {string} basePath (optional) folder path inside of the `content` folder to search for the given slug
 * @returns
 */
function getDocBySlug(slug, basePath = "") {
    var _a, _b;
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // remove file extension from the slug
            if (!slug || typeof slug !== "string")
                throw Error(`Slug is not a string: ${slug}`);
            slug = (slug === null || slug === void 0 ? void 0 : slug.replace(/.md|.mdx|.html|.html$/, "")) || false;
            if (!slug)
                throw Error("No slug provided");
            // locate the document based on its `slug`
            const filePath = (0, fs_2.locateFilePath)({ slug, basePath });
            if (!filePath)
                return false;
            // load the doc and return it as requested
            const doc = yield loadAndParseDoc(filePath);
            // compute the href for the doc
            const href = computeHrefForDoc(doc, basePath);
            doc.href = href;
            if (!doc ||
                (((_a = doc === null || doc === void 0 ? void 0 : doc.meta) === null || _a === void 0 ? void 0 : _a.draft) === true && ((_b = process.env) === null || _b === void 0 ? void 0 : _b.NODE_ENV) === "production"))
                return false;
            return doc;
        }
        catch (err) {
            console.warn("Unable to locate document:", slug);
            console.warn(err);
        }
        return false;
    });
}
exports.getDocBySlug = getDocBySlug;
/**
 * Compute the absolute href of a given doc
 * @param {object} doc full doc or doc.meta data object
 * @param {string} basePath string of the base bath used to locate the doc
 * @returns string of the absolute href for the given doc
 */
function computeHrefForDoc(doc, basePath) {
    //
    let href = doc.path.substring(doc.path.indexOf(basePath));
    //
    href = href.substring(0, href.indexOf(doc.slug));
    //
    href = `/${href}${doc.slug}`;
    return href;
}
exports.computeHrefForDoc = computeHrefForDoc;
/**
 * Retrieve a markdown document's front matter meta, parsed and ready to go
 * @param {string} slug slug of the file name to locate
 * @param {string} basePath (optional) folder path inside of the `content` folder to search for the given slug
 * @returns
 */
function getDocMetaBySlug(slug, basePath = "") {
    return __awaiter(this, void 0, void 0, function* () {
        try {
            const doc = yield getDocBySlug(slug, basePath);
            if (!doc || typeof doc !== "object")
                throw Error("Unable to locate doc");
            // clear the content
            doc.content = "";
            return doc || false;
        }
        catch (err) {
            console.warn("Unable to locate document:", slug);
            console.warn(err);
        }
        return false;
    });
}
exports.getDocMetaBySlug = getDocMetaBySlug;
/**
 * Retrieve a listing of markdown documents from the given `searchPath` directory, parsed and ready to go
 * @param {string} searchPath base relative path of documents to locate
 * @param {object} options custom options to use on the docs
 * @returns `array` of documents located inside the `searchPath`
 */
function getDocsByPath(searchPath = "", options = {}) {
    var _a, _b, _c;
    return __awaiter(this, void 0, void 0, function* () {
        // define the default options, and slice in the custom client supplied ones
        options = Object.assign({ filters: false, metaOnly: true, hideDrafts: false, draftsInProd: false }, options);
        // define the base array of docs
        let docs = [];
        try {
            // crawl the `searchPath` for all the documents, and parse them
            const files = (0, fs_2.crawlForFiles)(searchPath, true);
            // load and parse each of the located docs
            for (let i = 0; i < files.length; i++) {
                // attempt to load the doc's meta info
                try {
                    const doc = yield loadAndParseDoc(files[i], options.metaOnly);
                    if (doc) {
                        if (((_a = doc === null || doc === void 0 ? void 0 : doc.meta) === null || _a === void 0 ? void 0 : _a.draft) === true && options.hideDrafts)
                            continue;
                        else if (((_b = doc === null || doc === void 0 ? void 0 : doc.meta) === null || _b === void 0 ? void 0 : _b.draft) !== true ||
                            ((_c = process.env) === null || _c === void 0 ? void 0 : _c.NODE_ENV) !== "production")
                            docs.push(doc);
                    }
                }
                catch (err) {
                    console.warn("Unable to parse doc:", files[i]);
                }
            }
            // sort the docs by their dates (making the newest first)
            docs = (0, formatters_1.sortByPriorityDate)(docs, "desc");
            // pre filter the docs before returning (when desired)
            if ((options === null || options === void 0 ? void 0 : options.filters) && typeof options.filters === "object")
                docs = filterDocs(docs, options.filters);
            return docs;
        }
        catch (err) {
            console.warn("Unable to locate path:", path_1.default);
            console.warn(err);
        }
        // always returns an array, hopefully with docs!
        return docs;
    });
}
exports.getDocsByPath = getDocsByPath;
/**
 * Load and parse a the file from the given `filePath`, making it ready for the frontend.
 * @param {string} filePath path to the file to load and parse
 * @param {boolean} metaOnly whether or not to only parse/return the meta data
 * @returns
 */
function loadAndParseDoc(filePath, metaOnly = false) {
    var _a, _b, _c, _d;
    return __awaiter(this, void 0, void 0, function* () {
        try {
            // read the file from the local file system
            const stats = fs_1.default.statSync(path_1.default.join(filePath));
            let file = fs_1.default.readFileSync(path_1.default.join(filePath), "utf-8");
            file = (0, gray_matter_1.default)(file);
            // extract the desired attributes and values
            // construct the parsed 'doc' to return
            const doc = {
                path: filePath,
                href: null,
                slug: null,
                meta: (file === null || file === void 0 ? void 0 : file.data) || {},
                content: metaOnly ? null : file === null || file === void 0 ? void 0 : file.content,
            };
            // always define the `createdAt` date
            doc.meta.createdAt = luxon_1.DateTime.fromJSDate(stats.birthtime).toString();
            // process the front matter supplied `date`
            if ((_a = doc.meta) === null || _a === void 0 ? void 0 : _a.date)
                doc.meta.date = new Date(doc.meta.date).toISOString();
            // process the front matter supplied `updatedAt`
            if (((_b = doc === null || doc === void 0 ? void 0 : doc.meta) === null || _b === void 0 ? void 0 : _b.updatedAt) && ((_c = doc === null || doc === void 0 ? void 0 : doc.meta) === null || _c === void 0 ? void 0 : _c.updatedAt) !== false)
                doc.meta.updatedAt = new Date(doc.meta.updatedAt).toISOString();
            else
                doc.meta.updatedAt = luxon_1.DateTime.fromJSDate(stats.mtime).toString();
            // TODO: generate the SEO details?
            // TODO: generate the date stamps to be used in the file
            // generate and store the slug
            // TODO: handle user specified slugs in the front matter
            doc.slug = (0, formatters_1.generateSlug)(doc);
            doc.meta.slug = doc.slug;
            // always convert any found `tags` to an array of tags
            if ((_d = doc.meta) === null || _d === void 0 ? void 0 : _d.tags) {
                if (!Array.isArray(doc.meta.tags))
                    doc.meta.tags = doc.meta.tags
                        .toString()
                        .split(",")
                        .map((tag) => tag.trim());
                if (!Array.isArray(doc.meta.tags))
                    doc.meta.tags = null;
            }
            // TODO: this does not parse MDX files, or actually check to make sure the file extension is .MD
            return doc;
        }
        catch (err) {
            console.warn("Unable to parse file");
            // throw err;
        }
        // default return
        return false;
    });
}
exports.loadAndParseDoc = loadAndParseDoc;
/**
 * Filter a listing of documents by the provided `filter` criteria on the `doc.meta` data
 * @param {array} docs array of documents to be filtered
 * @param {object} filters criteria to filter by
 * @param {number} limit max number of filtered documents to return
 * @returns
 */
function filterDocs(docs, filters, limit = 0) {
    try {
        // parse each of the provided filters
        for (const [key, value] of Object.entries(filters)) {
            // do NOT parse arrays!
            if (Array.isArray(value))
                return false;
            // filter the provided `docs` listing for the current working filter (aka `key`)
            docs = docs === null || docs === void 0 ? void 0 : docs.filter((item) => {
                // console.log(key);
                var _a, _b, _c, _d;
                // TODO: allow nested parsing
                // while (typeof value === "object" && item[key] !== undefined) {
                //   // if (typeof value === "object") {
                //   console.log(value);
                //   console.log(item);
                //   item = item[key]
                //   // }
                // }
                // auto select the `meta` object from each document
                item = item === null || item === void 0 ? void 0 : item.meta;
                // check for exact match for the simple types of `boolean` and `string`
                if (typeof value === "boolean" || typeof value === "string")
                    return (item === null || item === void 0 ? void 0 : item[key]) === value;
                // parse each of the comparison operators
                else if (typeof value === "object") {
                    if (value === null || value === void 0 ? void 0 : value.eq)
                        return (item === null || item === void 0 ? void 0 : item[key]) === value.eq;
                    else if (value === null || value === void 0 ? void 0 : value.neq)
                        return (item === null || item === void 0 ? void 0 : item[key]) !== value.neq;
                    else if (value === null || value === void 0 ? void 0 : value.gt)
                        return (item === null || item === void 0 ? void 0 : item[key]) > value.gt;
                    else if (value === null || value === void 0 ? void 0 : value.gte)
                        return (item === null || item === void 0 ? void 0 : item[key]) >= value.gte;
                    else if (value === null || value === void 0 ? void 0 : value.lt)
                        return (item === null || item === void 0 ? void 0 : item[key]) < value.lt;
                    else if (value === null || value === void 0 ? void 0 : value.lte)
                        return (item === null || item === void 0 ? void 0 : item[key]) <= value.lte;
                    else if (value === null || value === void 0 ? void 0 : value.startsWith)
                        return (_a = item === null || item === void 0 ? void 0 : item[key]) === null || _a === void 0 ? void 0 : _a.startsWith(value.startsWith.toString());
                    else if (value === null || value === void 0 ? void 0 : value.endsWith)
                        return (_b = item === null || item === void 0 ? void 0 : item[key]) === null || _b === void 0 ? void 0 : _b.endsWith(value.endsWith.toString());
                    else if ((_c = value === null || value === void 0 ? void 0 : value.contains) === null || _c === void 0 ? void 0 : _c.toString())
                        return ((_d = item === null || item === void 0 ? void 0 : item[key]) === null || _d === void 0 ? void 0 : _d.indexOf(value.contains.toString())) >= 0;
                    else
                        return false;
                }
            });
        }
        // return false when no documents were found
        if (!docs || !(docs === null || docs === void 0 ? void 0 : docs.length))
            return false;
        // return the final filtered (and limited) results
        return limit ? docs === null || docs === void 0 ? void 0 : docs.slice(0, limit) : docs;
    }
    catch (err) {
        console.warn("Unable to filter documents:", filters);
        console.warn(err);
    }
    return false;
}
exports.filterDocs = filterDocs;
//# sourceMappingURL=docs.js.map